@@ -95,7 +95,7 @@
  */
 #define VMMOUSE_MAJOR_VERSION 12
 #define VMMOUSE_MINOR_VERSION 6
-#define VMMOUSE_PATCHLEVEL 1
+#define VMMOUSE_PATCHLEVEL 2
 #define VMMOUSE_DRIVER_VERSION \
    (VMMOUSE_MAJOR_VERSION * 65536 + VMMOUSE_MINOR_VERSION * 256 + VMMOUSE_PATCHLEVEL)
 #define VMMOUSE_DRIVER_VERSION_STRING \
@@ -310,6 +310,9 @@ VMMousePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
    if (!mPriv) {
       return NULL;
    }
+
+   mPriv->relative = TRUE;
+
    /*
     * try to enable vmmouse here
     */
@@ -603,8 +606,7 @@ VMMousePostEvent(InputInfoPtr pInfo, int buttons, int dx, int dy, int dz, int dw
  *
  * FlushButtons --
  *
- * 	FlushButtons -- send button up events for sanity. It is called
- *	during DEVICE_ON in VMMouseDeviceControl
+ * 	FlushButtons -- reset button states.
  *
  * Results:
  * 	None
@@ -618,18 +620,8 @@ VMMousePostEvent(InputInfoPtr pInfo, int buttons, int dx, int dy, int dz, int dw
 static void
 FlushButtons(MouseDevPtr pMse)
 {
-
-    /* If no button down is pending xf86PostButtonEvent()
-     * will discard them. So we are on the safe side. */
-
-    int i, blocked;
-
     pMse->lastButtons = 0;
-
-    blocked = xf86BlockSIGIO ();
-    for (i = 1; i <= 5; i++)
-	xf86PostButtonEvent(pMse->device,0,i,0,0,0);
-    xf86UnblockSIGIO (blocked);
+    pMse->lastMappedButtons = 0;
 }
 
 
@@ -854,8 +846,6 @@ VMMouseDeviceControl(DeviceIntPtr device, int mode)
 		  return FALSE;
 	       } else {
 		  mPriv->vmmouseAvailable = TRUE;
-		  VMMouseClient_RequestAbsolute();
-		  mPriv->relative = FALSE;
 		  xf86Msg(X_INFO, "VMWARE(0): vmmouse enabled\n");
 	       }
 	    }
@@ -876,6 +866,7 @@ VMMouseDeviceControl(DeviceIntPtr device, int mode)
 	 if( mPriv->vmmouseAvailable ) {
 	    VMMouseClient_Disable();
 	    mPriv->vmmouseAvailable = FALSE;
+            mPriv->relative = TRUE;
 	 }
 
 	 xf86RemoveEnabledDevice(pInfo);
@@ -925,6 +916,12 @@ VMMouseReadInput(InputInfoPtr pInfo)
    pMse = pInfo->private;
    mPriv = pMse->mousePriv;
 
+   if (mPriv->relative) {
+      VMMouseClient_RequestAbsolute();
+      mPriv->relative = FALSE;
+      xf86Msg(X_INFO, "VMWARE(0): vmmouse enable absolute mode\n");
+   }
+
    /*
     * First read the bytes in input device to clear the regular PS/2 fd so
     * we don't get called again.
